/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  Copyright © 2007 Pierre Habouzit
 */

#include <lib-lib/lib-lib.h>
#include <lib-lua/lib-lua.h>

#include "../alias.h"
#include "../mutt.h"
#include "../charset.h"
#include "../buffy.h"
#include "../crypt.h"
#include <lib-sys/mutt_ssl.li>

static lua_State *L;

void luaM_initialize(void)
{
    static const luaL_Reg lualibs[] = {
        {"",              luaopen_base},
        {LUA_OSLIBNAME,   luaopen_os},
        {LUA_LOADLIBNAME, luaopen_package},
        {LUA_TABLIBNAME,  luaopen_table},
        {LUA_IOLIBNAME,   luaopen_io},
        {LUA_STRLIBNAME,  luaopen_string},
        {LUA_MATHLIBNAME, luaopen_math},
        {LUA_DBLIBNAME,   luaopen_debug},
        {"MCore",         luaopen_MCore},
        {"MTransport",    luaopen_MTransport},
        {"MAlias",        luaopen_MAlias},
        {"MCharset",      luaopen_MCharset},
        {"Mime",          luaopen_Mime},
        {"Buffy",         luaopen_Buffy},
        {"mod_ssl",       luaopen_mod_ssl},
        {"mod_crypt",     luaopen_mod_crypt},
    };

    int i;

    L = lua_open();
    for (i = 0; i < countof(lualibs); i++) {
        lua_pushcfunction(L, lualibs[i].func);
        lua_pushstring(L, lualibs[i].name);
        lua_call(L, 1, 0);
    }
}

void luaM_shutdown(void)
{
    lua_close(L);
}

lua_State *luaM_getruntime(void)
{
    return L;
}

int luaM_dofile(const char *filename)
{
    return luaL_dofile(L, filename);
}

int luaM_wrap(void (*errfun)(const char *fmt, ...), int status)
{
    if (status) {
        (*errfun)("-[lua]-: %s\n", lua_tostring(L, -1));
        lua_pop(L, 1);
    }
    return status;
}


quadopt_t luaM_checkquadopt(lua_State *Ls, int narg)
{
    const char *s;
    int i;

    if (lua_type(Ls, narg) == LUA_TSTRING) {
        s = lua_tostring(Ls, narg);
        switch (mlua_which_token(s, -1)) {
          case LTK_YES:     return M_YES;
          case LTK_NO:      return M_NO;
          case LTK_ASK_YES: return M_ASKYES;
          case LTK_ASK_NO:  return M_ASKNO;
          default:
            break;
        }
    }

    i = luaL_checkinteger(Ls, narg);
    if (i & ~3) {
        s = lua_pushfstring(Ls, "int in [0-3] expected, got %d", i);
        return luaL_argerror(Ls, narg, s);
    }
    return i;
}

void luaM_pushquadopt(lua_State *Ls, int val)
{
    switch (val) {
      case M_YES:    return lua_pushstring(Ls, "yes");
      case M_NO:     return lua_pushstring(Ls, "no");
      case M_ASKYES: return lua_pushstring(Ls, "ask-yes");
      case M_ASKNO:  return lua_pushstring(Ls, "ask-no");
      default:       return lua_pushnil(Ls);
    }
}

const char *luaM_checkrx(lua_State *Ls, int narg)
{
    const char *s = luaL_checkstring(Ls, narg);
    char buf[STRING];

    if (rx_validate(s, buf, ssizeof(buf))) {
        s = lua_pushfstring(Ls, "invalid regexp: `%s'", buf);
        luaL_argerror(Ls, narg, s);
        return NULL;
    }

    return s;
}

rx_t *luaM_rxnew(const char *val)
{
    if (m_strisempty(val))
        val = ".";

    return rx_compile(val, mutt_which_case(val));
}

char *luaM_pathnew(const char *val)
{
    char path[PATH_MAX];
    _mutt_expand_path(path, sizeof(path), val, 0);
    return m_strdup(path);
}


void luaM_pushaddr(lua_State *Ls, address_t *addr)
{
    char s[HUGE_STRING] = "";
    rfc822_addrcat(s, sizeof(s), addr, 0);
    lua_pushstring(Ls, s);
}
